require 'net/ssh'

module Net
  module SSH
    module Script
      # Net::SSH::Connection::Session wrapper to capture
      # status/stdout/stderr produced by a command.
      class Command
        attr_reader :output, :error

        def self.[](command)
          new command
        end

        def initialize(command_string)
          @_cmd = command_string
          @_done = @_success = false
          @output = @error = ''
        end

        def run(ssh)
          channel = ssh.open_channel do |ch|
            # In case when we want rerun a command
            @_done = @_success = false
            exec_command @_cmd, ch
          end

          channel.wait

          ok?
        end

        def done?
          @_done
        end

        def inspect
          case status
          when :ok
            "#{status}: #{@_cmd}\nOutput:\n#{@output}\n"
          when :error
            "#{status}: #{@_cmd}\nError:\n#{@error}\n"
          else
            "#{status}: #{@_cmd}"
          end
        end

        def status
          return :pending unless done?
          return :failed unless @_success
          return :error unless @error.empty?
          :ok
        end

        def to_s
          @_cmd
        end

        [:pending?, :failed?, :error?, :ok?].each do |helper|
          define_method helper do
            status == helper.to_s.sub('?', '').to_sym
          end
        end

        private

        def listen_channel(chan)
          chan.on_close { @_done = true }
          chan.on_data { |_, data| @output << data }
          chan.on_extended_data do |_, type, data|
            @error << "(#{type}):#{data}\n"
          end
        end

        def exec_command(command, chan)
          chan.exec command do |ch, success|
            @_success = success

            unless success
              @_done = true
              return false
            end

            listen_channel ch
          end
        end
      end
    end
  end
end
